@@ -31,7 +31,8 @@ include_once(SM_PATH . 'plugins/calendar_sql_backend/functions.php');
 function cal_sql_get_all_owned_calendars_do($user, $userType)
 {
 
-   global $all_calendars_query, $owned_calendars_query, $all_calendars_of_type_query, $color;
+   global $all_calendars_query, $owned_calendars_query, $all_calendars_of_type_query, 
+          $wildcard_calendar_owners_query, $color;
 
 
    // get database connection
@@ -94,6 +95,34 @@ function cal_sql_get_all_owned_calendars_do($user, $userType)
       foreach ($ownedIDs as $ID) $ownedCalIDs[] = $ID[0];
 
 
+      // get calendars with owner names that have wildcards in them
+      //
+      $sql = $wildcard_calendar_owners_query;
+
+
+      $wildcardOwners = $db->getAll($sql);
+
+
+      // check for database errors
+      //
+      if (DB::isError($wildcardOwners))
+      {
+         $msg = $wildcardOwners->getMessage();
+         plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_owned_calendars_do): could not query owners with wildcards - ' . $msg, $color);
+         exit;
+      }
+
+
+      // figure out which ones match current user
+      //
+      foreach ($wildcardOwners as $owner) 
+      {
+         if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                        strtoupper($owner[0])) . '$/', strtoupper($user)))
+            $ownedCalIDs[] = $owner[1];
+      }
+
+
       // unpriveleged users just get their private/personal
       // calendars... they should never own anything but, 
       // except we'll make sure...  just in case
@@ -214,7 +243,11 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
 {
 
    global $all_owned_calendars_of_type_query, $all_readable_calendars_of_type_query, 
-          $all_writeable_calendars_of_type_query, $color;
+          $all_writeable_calendars_of_type_query, 
+          $wildcard_calendar_owners_of_type_query,
+          $wildcard_calendar_readers_of_type_query, 
+          $wildcard_calendar_writers_of_type_query, $color;
+          
 
 
    // get database connection
@@ -222,6 +255,7 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    $db = cal_get_database_connection();
 
 
+   // -------------------------------
    // get owned private cals
    //
    $sql = $all_owned_calendars_of_type_query;
@@ -247,6 +281,37 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($ownedPrivateIDs as $ID) $ownedPrivateCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get private calendars with owner names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_owners_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_PERSONAL, $sql);
+
+
+   $wildcardOwners = $db->getAll($sql);
+
+
+   // check for database errors
+   //
+   if (DB::isError($wildcardOwners))
+   {
+      $msg = $wildcardOwners->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query owners with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+
+   // figure out which ones match current user
+   //
+   foreach ($wildcardOwners as $owner)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($owner[0])) . '$/', strtoupper($user)))
+         $ownedPrivateCalIDs[] = $owner[1];
+   }
+
+
+   // -------------------------------
    // get readable private cals
    //
    $sql = $all_readable_calendars_of_type_query;
@@ -272,6 +337,37 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($readablePrivateIDs as $ID) $readablePrivateCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get private calendars with reader names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_readers_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_PERSONAL, $sql);
+
+
+   $wildcardReaders = $db->getAll($sql);
+
+
+   // check for database errors
+   //
+   if (DB::isError($wildcardReaders))
+   {
+      $msg = $wildcardReaders->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query readers with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+
+   // figure out which ones match current user
+   //
+   foreach ($wildcardReaders as $reader)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($reader[0])) . '$/', strtoupper($user)))
+         $readablePrivateCalIDs[] = $reader[1];
+   }
+
+
+   // -------------------------------
    // get writeable private cals
    //
    $sql = $all_writeable_calendars_of_type_query;
@@ -297,6 +393,39 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($writeablePrivateIDs as $ID) $writeablePrivateCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get private calendars with writer names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_writers_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_PERSONAL, $sql);
+
+
+   $wildcardWriters = $db->getAll($sql);
+
+
+   // check for database errors
+   //
+   if (DB::isError($wildcardWriters))
+   {
+      $msg = $wildcardWriters->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query writers with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+
+   // figure out which ones match current user
+   //
+   foreach ($wildcardWriters as $writer)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($writer[0])) . '$/', strtoupper($user)))
+         $writeablePrivateCalIDs[] = $writer[1];
+   }
+
+
+   // -------------------------------
+   // wrap up private calendars
+   //
    $privateCalIDs = array_unique(array_merge($writeablePrivateCalIDs, 
                                              $readablePrivateCalIDs, 
                                              $ownedPrivateCalIDs));
@@ -308,6 +437,7 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    $privateCalIDs = array_diff($privateCalIDs, array($privateCalID));
 
 
+   // -------------------------------
    // get owned shared cals
    //
    $sql = $all_owned_calendars_of_type_query;
@@ -334,6 +464,37 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($ownedSharedIDs as $ID) $ownedSharedCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get shared calendars with owner names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_owners_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_SHARED, $sql);
+
+
+   $wildcardOwners = $db->getAll($sql);
+
+
+   // check for database errors
+   //
+   if (DB::isError($wildcardOwners))
+   {
+      $msg = $wildcardOwners->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query owners with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+
+   // figure out which ones match current user
+   //
+   foreach ($wildcardOwners as $owner)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($owner[0])) . '$/', strtoupper($user)))
+         $ownedSharedCalIDs[] = $owner[1];
+   }
+
+
+   // -------------------------------
    // get readable shared cals
    //
    $sql = $all_readable_calendars_of_type_query;
@@ -359,6 +520,37 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($readableSharedIDs as $ID) $readableSharedCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get shared calendars with reader names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_readers_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_SHARED, $sql);
+
+
+   $wildcardReaders = $db->getAll($sql);
+
+
+   // check for database errors
+   //
+   if (DB::isError($wildcardReaders))
+   {
+      $msg = $wildcardReaders->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query readers with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+
+   // figure out which ones match current user
+   //
+   foreach ($wildcardReaders as $reader)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($reader[0])) . '$/', strtoupper($user)))
+         $readableSharedCalIDs[] = $reader[1];
+   }
+
+
+   // -------------------------------
    // get writeable shared cals
    //
    $sql = $all_writeable_calendars_of_type_query;
@@ -384,6 +576,39 @@ function cal_sql_get_all_accessible_calendars_do($user, $domain)
    foreach ($writeableSharedIDs as $ID) $writeableSharedCalIDs[] = $ID[0];
 
 
+   // -------------------------------
+   // get shared calendars with writer names that have wildcards in them
+   //
+   $sql = $wildcard_calendar_writers_of_type_query;
+   $sql = str_replace('%1', SM_CAL_TYPE_SHARED, $sql);
+
+
+   $wildcardWriters = $db->getAll($sql);
+   
+   
+   // check for database errors
+   //
+   if (DB::isError($wildcardWriters))
+   {
+      $msg = $wildcardWriters->getMessage();
+      plain_error_message('ERROR IN CALENDAR SQL BACKEND (cal_sql_get_all_accessible_calendars_do): could not query writers with wildcards - ' . $msg, $color);
+      exit;
+   }
+
+   
+   // figure out which ones match current user
+   //
+   foreach ($wildcardWriters as $writer)
+   {
+      if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'),
+                     strtoupper($writer[0])) . '$/', strtoupper($user)))
+         $writeableSharedCalIDs[] = $writer[1];
+   }  
+
+
+   // -------------------------------
+   // wrap up shared calendars
+   //
    $calIDs = array_unique(array_merge($writeableSharedCalIDs, $readableSharedCalIDs, 
                                       $ownedSharedCalIDs, $privateCalIDs));
 
